from django import template
from django.conf import settings
from django.contrib.sites.models import Site
from django.contrib.sites.shortcuts import get_current_site

from ..settings import epayco_settings

register = template.Library()


@register.inclusion_tag('epayco_web_checkout_button.html')
def render_checkout(amount, name, tax=0, tax_base=0, description='', currency='COP', country_code='co',
                    external=True, extra1='', extra2='', extra3='', extra4='', extra5='', extra6='', extra7='',
                    extra8='', extra9='', extra10='', same_site=True, request=None, response_url=None, **kwargs):
    if description == '':
        description = name
    if same_site:
        if request:
            domain = get_current_site(request).domain
        else:
            try:
                domain = Site.objects.get(pk=settings.SITE_ID).domain
            except:
                domain = ''
    else:
        domain = ''
    return {
        'public_key': epayco_settings.PUBLIC_KEY,
        'amount': amount, 'tax': tax, 'tax_base': tax_base,
        'name': name, 'description': description,
        'currency': currency, 'country_code': country_code,
        'test': '{}'.format(epayco_settings.TEST).lower(), 'external': '{}'.format(external).lower(),
        'extra1': extra1, 'extra2': extra2, 'extra3': extra3, 'extra4': extra4,
        'extra5': extra5, 'extra6': extra6, 'extra7': extra7, 'extra8': extra8,
        'extra9': extra9, 'extra10': extra10,
        'response_url': response_url if response_url else epayco_settings.RESPONSE_URL, 'confirmation_url': epayco_settings.CONFIRMATION_URL,

        'same_site': same_site,
        'domain': domain
    }



import hashlib

import pyepayco.epayco as Epayco
import requests

from .settings import epayco_settings


def get_signature(cust_id_client, p_key, ref_payco, transaction_id, amount, currency_code):
    """
        Genera el signature requerido por ePayco para verificar una confirmación.
    """
    signature = '{}^{}^{}^{}^{}^{}'.format(cust_id_client, p_key, ref_payco,
                                           transaction_id, amount, currency_code)
    return hashlib.sha256(signature.encode('utf')).hexdigest()


def validate_response_code(ref_payco, request=None):
    from .models import PaymentConfirmation
    options = {'apiKey': epayco_settings.PUBLIC_KEY,
               'privateKey': epayco_settings.PRIVATE_KEY,
               'test': epayco_settings.TEST,
               'lenguage': 'ES'}
    check_response_ref_url = 'https://api.secure.payco.co/validation/v1/reference/{}'.format(ref_payco)
    response = requests.get(check_response_ref_url)
    r = response.json()
    if 'success' in r.keys() and r['success'] == True:
        try:
            ref_payco = r['data']['x_ref_payco']
        except:
            raise
            pass
    qs = PaymentConfirmation.objects.filter(ref_payco__iexact=ref_payco)
    if not qs.exists():
        epayco = Epayco.Epayco(options)
        response = epayco.cash.get(ref_payco)  # Validate the reference
        if response['success'] == True:
            if epayco_settings.CONFIRMATION_URL.startswith('http'):
                url = epayco_settings.CONFIRMATION_URL
            else:
                url = '{}://{}{}'.format('https' if epayco_settings.IS_SECURE else 'http',
                                         request.get_host(), epayco_settings.CONFIRMATION_URL)
            r2 = requests.post(url, data=response['data'])
            if r2.status_code == 405:
                raise Exception('There seems the be an error reaching the confirmation URL.'
                               ' Please make sure you are making good use of the "IS_SECURE" setting.')
            obj = PaymentConfirmation.objects.filter(ref_payco__iexact=ref_payco).last()
            if obj:
                return {'valid_ref': True, 'existed': False, 'flag': obj.is_flagged, 'obj': obj}
            else:
                return {''}
        return {'valid_ref': False}
    elif qs.filter(flag=False).exists():
        obj = qs.filter(flag=False).first()
        return {'valid_ref': True, 'existed': True, 'flag': False, 'obj': obj}
    else:
        obj = qs.first()
        return {'valid_ref': True, 'existed': True, 'flag': True, 'obj': obj}
